<?php
/**
 * @file
 * Contains the ISunMailerContentEventHandler interface.
 */

/**
 * Updates subscriptions when the content of a newsletter changes.
 */
interface ISunMailerContentEventHandler {
  /**
   * Invoked when a form to add a section is built.
   *
   * @return
   *   Form elements to embed into the form to add a section.
   */
  public function sectionAddForm();

  /**
   * Invoked when a form to add a section is validated.
   *
   * @param $edit
   *   Submitted values for the embedded form elements.
   *
   * @return
   *   Form error or null if no form error exists.
   */
  public function sectionAddValidate(&$edit);

  /**
   * Invoked when a form to add a section is submitted, after the section has been added.
   *
   * @param $section
   *   Section that was added
   * @param
   *   Submitted values for the embedded form elements.
   */
  public function sectionAddSubmit($section, &$edit);

  /**
   * Invoked when a section is about to be deleted.
   *
   * @param $section
   *   Section to be deleted.
   */
  public function sectionDelete($section);

  /**
   * Invoked when a newsletter is about to be deleted.
   *
   * @param $newsletter
   *   Newsletter to be deleted.
   */
  public function newsletterDelete($newsletter);
}