<?php
/**
 * @file
 * Contains the ISunMailerSubscriptionAggregator interface.
 */

/**
 * Aggregates subscription information needed to send a newsletter or a custom email.
 */
interface ISunMailerSubscriptionAggregator {
  /**
   * Gets all subscriptions for a newsletter.
   *
   * @param nlid
   *   Newsletter's ID.
   *
   * @return
   *   For each email address, a list of IDs for sections subscribed to.
   */
  public function getNewsletterSubscriptions($nlid);

  /**
   * Gets all subscriptions for a newsletter, grouping subscribers with a common subscription.
   *
   * @param nlid
   *   Newsletter's ID.
   *
   * @return
   *   For each comma-delimited list of section IDs, a list of email adddresses subscribed to those sections.
   */
  public function getGroupedNewsletterSubscriptions($nlid);

  /**
   * Gets all subscribers for one or more newsletters.
   *
   * @param $nlids
   *   (Optional) IDs of newsletters to get subscribers for. Omit for all newsletters.
   *
   * @return
   *   List of email addresses subscribed to one or more of those newsletters.
   */
  public function getAllSubscribers($nlids = array());
}
