<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output method="html"/>
  
  <xsl:template match="/newsletter">
    <html>
      <body>
        <table width="600" style="border: 1px lightgray solid; padding: 5px; background-color: white">
          <xsl:apply-templates select="header"/>
          <xsl:apply-templates select="sections/section"/>
          <xsl:apply-templates select="footer"/>
        </table>
      </body>
    </html>
  </xsl:template>
  
  <xsl:template match="header">
    <tr>
      <td style="border-bottom: 1px solid black; text-align: center; padding 5px 0px">
        <xsl:choose> 
          <xsl:when test="logo">
            <a>
              <xsl:attribute name="href">
                <xsl:value-of select="link" disable-output-escaping="yes"/>
              </xsl:attribute>
              <img style="border-width: 0px">
                <xsl:attribute name="src">
                  <xsl:value-of select="logo" disable-output-escaping="yes"/>
                </xsl:attribute>
                <xsl:attribute name ="alt">
                  <xsl:value-of select="title" disable-output-escaping="yes"/>
                </xsl:attribute>
              </img>
            </a>
          </xsl:when>
          <xsl:otherwise>
            <h2>
              <a>
                <xsl:attribute name="href">
                  <xsl:value-of select="link" disable-output-escaping="yes"/>
                </xsl:attribute>
                <xsl:value-of select="title" disable-output-escaping="yes"/>
              </a>                      
            </h2>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:if test="date">
          <p style="font-size: small">
            <strong>
              <xsl:value-of select="date" disable-output-escaping="yes"/>
            </strong>
          </p>
        </xsl:if>
      </td>
    </tr>
    <xsl:if test="message">
      <tr>
        <td style="border-bottom: 1px solid black; text-align: center; padding: 5px 0px">
          <p>
            <xsl:value-of select="message" disable-output-escaping="yes"/>
          </p>
        </td>
      </tr>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="sections/section">
    <xsl:if test="title">
      <tr style="border: 0px; padding: 0px; margin: 0px">
        <th style="border-top: 1px solid black; border-bottom: 5px solid black; padding: 5px 0px; font-size: large">
          <p>
            <xsl:value-of select="title" disable-output-escaping="yes"/>
          </p>
        </th> 
      </tr>
    </xsl:if>
    <xsl:apply-templates select="content"/>
  </xsl:template>
  
  <xsl:template match="content">
    <tr style="border: 0px; padding: 0px; margin: 0px">
      <td style="border-bottom: 1px solid black; padding: 5px 0px">
        <p>
          <strong>
            <a>
              <xsl:attribute name="href">
                <xsl:value-of select="link" disable-output-escaping="yes"/>
              </xsl:attribute>
              <xsl:value-of select="title" disable-output-escaping="yes"/>
            </a>
          </strong>
          <br />
          <xsl:if test="author">
            <a>
              <xsl:attribute name="href">
                <xsl:value-of select="author-link" disable-output-escaping="yes"/>
              </xsl:attribute>
              <xsl:value-of select="author" disable-output-escaping="yes"/>
            </a>
            <br />
          </xsl:if>
          <em>
            <xsl:value-of select="date" disable-output-escaping="yes"/>
          </em>
          <br/>
          <xsl:value-of select="teaser" disable-output-escaping="yes"/>
        </p>
      </td>
    </tr>
  </xsl:template>         
          
  <xsl:template match="footer">
    <tr>
      <td style="border: 0px; padding: 5px 0px">
        <p>To change your subscription or unsubscribe, please visit
          <a>
            <xsl:attribute name="href">
              <xsl:value-of select="subscription-link" disable-output-escaping="yes"/>
            </xsl:attribute>
            <xsl:value-of select="subscription-link" disable-output-escaping="yes"/>
          </a>.
        </p>
      </td>
    </tr>
  </xsl:template>
</xsl:stylesheet>
