<?php

/**
 * Builds the form for an overview of the newsletters.
 */
function sunmailer_content_overview_newsletters() {
  $base = 'admin/settings/sunmailer/content';
  $list = array();
  $form = array();
  $form['content']['#tree'] = TRUE;

  $newsletters = SunMailerNewsletter::loadAllNewsletters();
  $newsletters_delta = count($newsletters);
  foreach ($newsletters as $nlid => $newsletter) {
    $list[$nlid] = array();
    $form['content']['newsletter'][$nlid] = array(
      'title' => array(
        '#type' => 'markup',
        '#value' => $newsletter->getTitle(),
      ),
      'edit' => array(
        '#type' => 'markup',
        '#value' => l('edit', "$base/newsletter/$nlid/edit"),
      ),
      'delete' => array(
        '#type' => 'markup',
        '#value' => l('delete', "$base/newsletter/$nlid/delete"),
      ),
      'add_section' => array(
        '#type' => 'markup',
        '#value' => l('add section', "$base/section/add/$nlid"),
      ),
      'weight' => array(
        '#type' => 'weight',
        '#delta' => $newsletters_delta,
        '#default_value' => $newsletter->getWeight(),
      ),
    );

    $sections = SunMailerSection::loadNewsletterSections($nlid);
    $sections_delta = count($sections);
    foreach ($sections as $sid => $section) {
      $list[$nlid][] = $sid;
      $form['content']['section'][$sid] = array(
        'title' => array(
          '#type' => 'markup',
          '#value' => $section->getTitle(),
        ),
        'edit' => array(
          '#type' => 'markup',
          '#value' => l('edit', "$base/section/$sid/edit"),
        ),
        'delete' => array(
          '#type' => 'markup',
          '#value' => l('delete', "$base/section/$sid/delete"),
        ),
        'weight' => array(
          '#type' => 'weight',
          '#delta' => $sections_delta,
          '#default_value' => $section->getWeight(),
        ),
      );
    }
  }

  $form['list'] = array(
    '#type' => 'value',
    '#value' => $list,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );
  return $form;
}

/**
 * Submission callback for the newsletters overview form.
 */
function sunmailer_content_overview_newsletters_submit($form, &$form_state) {
  $values = $form_state['values'];
  $newsletters = SunMailerNewsletter::loadAllNewsletters();
  $sections = SunMailerSection::loadAllSections();
  foreach ($values['list'] as $nlid => $sids) {
    $newsletters[$nlid]->setWeight($values['content']['newsletter'][$nlid]['weight']);
    $newsletters[$nlid]->save();
    foreach ($sids as $sid) {
      $sections[$sid]->setWeight($values['content']['section'][$sid]['weight']);
      $sections[$sid]->save();
    }
  }
}

/**
 * Callback to generate a date in the specified format.
 */
function sunmailer_date_format() {
  switch ($_POST['date_type']) {
    case 'small':
    case 'medium':
    case 'large':
      $data = format_date(time(), $_POST['date_type']);
      break;
    case 'custom':
      $data = format_date(time(), 'custom', $_POST['date_format']);
      break;
  }
  print drupal_to_js(array('status' => TRUE, 'data' => $data));
  exit;
}

/**
 * Builds the form for a newsletter.
 */
function sunmailer_content_newsletter_form($form_state = NULL, $newsletter = NULL) {
  drupal_add_js(drupal_get_path('module', 'sunmailer') . '/sunmailer.jquery.js');
  if (!$newsletter) {
    $newsletter = SunMailerNewsletter::createNewsletter();
  }

  // Build the form.
  $form = array();
  $form['#attributes'] = array('enctype' => 'multipart/form-data');
  $form['nlid'] = array(
    '#type' => 'value',
    '#value' => $newsletter->getNewsletterId(),
  );

  $form['subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#description' => t('Subject line for the email - leave blank to use the site name'),
    '#default_value' => $newsletter->getSubject(),
  );

  $logo = $newsletter->getLogo();
  switch ($newsletter->getDateFormat()) {
    case '':
      $default_date_type = 'medium';
      $default_date_format = '';
      break;
    case 'small':
    case 'medium':
    case 'large':
      $default_date_type = $newsletter->getDateFormat();
      $default_date_format = '';
      break;
    default:
      $default_date_type = 'custom';
      $default_date_format = $newsletter->getDateFormat();
      break;
  }

  $form['header'] = array(
    '#type' => 'fieldset',
    '#title' => t('Header'),
    '#collapsible' => FALSE,
  );
  $form['header']['title'] = array(
    '#type' => 'fieldset',
    '#title' => t('Title'),
    '#collapsible' => FALSE,
  );
  $form['header']['title']['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#description' => t('Leave blank to use the site name'),
    '#default_value' => $newsletter->getTitle(),
  );
  $form['header']['title']['use_logo'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display a logo in place of the title'),
    '#description' => t('The title will still be used as the logo\'s alternate text'),
    '#default_value' => $newsletter->getLogoFileId(),
  );
  $form['header']['title']['logo'] = array(
    '#type' => 'file',
    '#title' => t('Logo'),
  );
  if ($logo) {
    $form['header']['title']['view_logo'] = array(
      '#type' => 'markup',
      '#prefix' => '<div id="view-logo-wrapper">',
      '#value' => t('Current logo: <a href="!path" target="_blank">!name</a>', array('!path' => '/' . $logo->filepath, '!name' => $logo->filename)),
      '#suffix' => '</div>',
    );
  }
  $form['header']['date'] = array(
    '#type' => 'fieldset',
    '#title' => t('Date'),
    '#collapsible' => FALSE,
  );
  $form['header']['date']['use_date'] = array(
    '#type' => 'checkbox',
    '#title' => 'Display the current date in the header',
    '#default_value' => $newsletter->getDateFormat(),
  );
  $form['header']['date']['date_type'] = array(
    '#type' => 'radios',
    '#title' => t('Date format'),
    '#options' => array(
      'small' => 'Small',
      'medium' => 'Medium',
      'large' => 'Large',
      'custom' => 'Custom',
    ),
    '#default_value' => $default_date_type,
    '#prefix' => '<div id="edit-date-type-wrapper">',
    '#suffix' => '</div>',
    '#ahah' => array(
      'path' => 'sunmailer/date_format',
      'wrapper' => 'display-date-format',
    ),
  );
  $form['header']['date']['date_format'] = array(
    '#type' => 'textfield',
    '#description' => t('Custom PHP <a href="http://www.php.net/manual/en/function.date.php" target="_blank">date format</a>'),
    '#default_value' => $default_date_format,
    '#ahah' => array(
      'path' => 'sunmailer/date_format',
      'wrapper' => 'view-date-format-wrapper',
    ),
  );
  $form['header']['date']['view_date_format'] = array(
    '#type' => 'markup',
    '#prefix' => '<div id="view-date-format-wrapper">',
    '#value' => format_date(time(), $default_date_type, $default_date_format),
    '#suffix' => '</div>'
  );

  // Generate the design options.
  $design_options = array();
  $alt_design_options = array(0 => t('(None)'));
  foreach (SunMailerDesign::LoadAllDesigns() as $did => $design) {
    $design_options[$did] = $design->getTitle();
    if (!$design->IsHtml()) {
      $alt_design_options[$did] = $design->getTitle();
    }
  }

  $form['design'] = array(
    '#type' => 'fieldset',
    '#title' => t('Design'),
    '#collapsible' => FALSE,
  );
  $form['design']['design'] = array(
    '#type' => 'select',
    '#title' => t('Design'),
    '#options' => $design_options,
    '#required' => TRUE,
    '#default_value' => $newsletter->getDesignId(),
  );
  $form['design']['alt_design'] = array(
    '#type' => 'select',
    '#title' => t('Alternate design'),
    '#description' => t('(Optional) Text design for email clients that do not support HTML'),
    '#options' => $alt_design_options,
    '#required' => TRUE,
    '#default_value' => $newsletter->getAlternateDesignId(),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  return $form;
}

/**
 * Validation callback for the newsletter form.
 */
function sunmailer_content_newsletter_form_validate($form, &$form_state) {
  if (!$form_state['values']['delete_logo']) {
    if ($file = file_save_upload('logo', array('file_validate_is_image' => array()), file_directory_path())) {
      $form_state['values']['logo_file'] = $file;
    }
  }
}

/**
 * Submission callback for the newsletter form.
 */
function sunmailer_content_newsletter_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $newsletter = $values['nlid'] ? SunMailerNewsletter::loadNewsletter($values['nlid']) : SunMailerNewsletter::createNewsletter();

  $newsletter->setSubject($values['subject']);
  $newsletter->setTitle($values['title']);
  if (!$values['use_logo']) {
    $newsletter->setLogoFileId(0);
  }
  elseif (isset($values['logo_file'])) {
    $newsletter->setLogoFileId($values['logo_file']->fid);
  }
  if ($values['use_date']) {
    switch ($values['date_type']) {
    case 'small':
    case 'medium':
    case 'large':
      $newsletter->setDateFormat($values['date_type']);
      break;
    case 'custom':
      $newsletter->setDateFormat($values['date_format']);
      break;
    }
  }
  else {
    $newsletter->setDateFormat('');
  }
  $newsletter->setDesignId($values['design']);
  $newsletter->setAlternateDesignId($values['alt_design']);
  $newsletter->save();

  // Display a message, redirect, and reset the menu.
  if ($values['nlid']) {
    drupal_set_message(t('Newsletter successfully edited'));
    $form_state['redirect'] = 'admin/settings/sunmailer/content';
  }
  else {
    drupal_set_message(t('Newsletter successfully added'));
  }
  menu_rebuild();
}

/**
 * Confirmation form to delete a newsletter.
 */
function sunmailer_content_newsletter_delete_confirm($form_state = NULL, $newsletter) {
  $form = array();
  $form['nlid'] = array(
    '#type' => 'value',
    '#value' => $newsletter->getNewsletterId(),
  );

  $question = t('Are you sure you want to delete the newsletter %title?', array('%title' => $newsletter->getTitle()));
  $description = t('This action will also delete this newsletter\'s sections and subscriptions to this newsletter.');
  return confirm_form($form, $question, 'admin/settings/sunmailer/content', $description, t('Delete'));
}

/**
 * Submission callback for newsletter deletion form.
 */
function sunmailer_content_newsletter_delete_confirm_submit($form, &$form_state) {
  // Delete the newsletter.
  $newsletter = SunMailerNewsletter::loadNewsletter($form_state['values']['nlid']);
  $content_event_handler = get_sunmailer_content_event_handler();
  $content_event_handler->newsletterDelete($newsletter);
  SunMailerNewsletter::deleteNewsletter($newsletter->getNewsletterId());

  // Display a message, redirect, and reset the menu.
  drupal_set_message(t('%title successfully deleted', array('%title' => $newsletter->getTitle())));
  $form_state['redirect'] = 'admin/settings/sunmailer/content';
  menu_rebuild();
}


/**
 * Autocomplete callback for a view and an associated display.
 */
function sunmailer_autocomplete_views($view_string = '', $display_string = '') {
  $views = views_get_all_views();
  $matches = array();
  foreach ($views as $view) {
    if ($view->disabled) {
      continue;
    }
    if ($view->base_table != 'node') {
      continue;
    }
    foreach ($view->display as $did => $display) {
      $match = "{$view->name}/$did";
      $string = $view_string . ($display_string ? "/$display_string" : '');
      $match_display = "$match ({$display->display_title})";
      if (strpos($match, $string) === 0) {
        $matches[$match] = $match_display;
      }
    }
  }
  drupal_json(array_slice($matches, 0, 10));
}

/**
 * Builds the form for a newsletter section.
 */
function sunmailer_content_section_form($form_state = NULL, $section = NULL, $newsletter = NULL) {
  if (!$section) {
    $section = SunMailerSection::createSection($newsletter->getNewsletterId());
  }

  $form = array();
  $form['sid'] = array(
    '#type' => 'value',
    '#value' => $section->getSectionId(),
  );
  $form['nlid'] = array(
    '#type' => 'value',
    '#value' => $section->getNewsletterId(),
  );

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#required' => TRUE,
    '#default_value' => $section->getTitle(),
  );
  $form['view_display'] = array(
    '#type' => 'textfield',
    '#title' => t('View'),
    '#description' => t('Format is "view/display" (e.g.: archive/default)'),
    '#required' => TRUE,
    '#default_value' => $section->getSectionId() ? "{$section->getViewName()}/{$section->getViewDisplay()}" : '',
    '#autocomplete_path' => 'sunmailer/autocomplete/views',
  );
  $form['args'] = array(
    '#type' => 'textfield',
    '#title' => t('Arguments'),
    '#description' => t('(Optional) Separate arguments with a / as though they were a URL path'),
    '#default_value' => $section->getViewArguments(),
  );

  if (!$section->getSectionId()) {
    $content_event_handler = get_sunmailer_content_event_handler();
    $form['subscription_model'] = $content_event_handler->sectionAddForm();
    $form['subscription_model']['#tree'] = TRUE;
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  return $form;
}

/**
 * Validation callback for the section form.
 */
function sunmailer_content_section_form_validate($form, &$form_state) {
  $values = $form_state['values'];
  $view_display = explode('/', $values['view_display']);
  $view = views_get_view($view_display[0]);
  if (!$view) {
    form_set_error('view_display', t('View does not exist.'));
    return;
  }
  if ($view->base_table != 'node') {
    form_set_error('view_display', t('View is not a node view.'));
    return;
  }
  if ($view->disabled) {
    form_set_error('view_display', t('View is disabled.'));
    return;
  }
  if (!array_key_exists($view_display[1], $view->display)) {
    form_set_error('view_display', t('View does not contain this display.'));
    return;
  }

  if (!$values['sid']) {
    $content_event_handler = get_sunmailer_content_event_handler();
    $error = $content_event_handler->sectionAddValidate($values['subscription_model']);
    if ($error) {
      form_set_error('subscription_model', $error);
    }
  }
}

/**
 * Submission callback for the section form.
 */
function sunmailer_content_section_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $section = $values['sid'] ? SunMailerSection::loadSection($values['sid']) : SunMailerSection::createSection($values['nlid']);

  // Save the section.
  $section->setTitle($values['title']);
  $view_display = explode('/', $values['view_display']);
  $section->setView($view_display[0], $view_display[1], $values['args']);
  $section->save();

  // Display a message and redirect.
  if ($values['sid']) {
    drupal_set_message(t('%title successfully edited', array('%title' => $section->getTitle())));
    $form_state['redirect'] = 'admin/settings/sunmailer/content';
  }
  else {
    $content_event_handler = get_sunmailer_content_event_handler();
    $content_event_handler->sectionAddSubmit($section, $values['subscription_model']);
    drupal_set_message(t('%title successfully added', array('%title' => $section->getTitle())));
  }
}

/**
 * Confirmation form to delete a newsletter section.
 */
function sunmailer_content_section_delete_confirm($form_state = NULL, $section) {
  $form = array();
  $form['sid'] = array(
    '#type' => 'value',
    '#value' => $section->getSectionId(),
  );

  $question = t('Are you sure you want to delete the section %title?', array('%title' => $section->getTitle()));
  $description = t('This action will also delete subscriptions to this section.');
  return confirm_form($form, $question, 'admin/settings/sunmailer/content', $description, t('Delete'));
}

/**
 * Submission callback for the section deletion form.
 */
function sunmailer_content_section_delete_confirm_submit($form, &$form_state) {
  // Delete the section.
  $section = SunMailerSection::loadSection($form_state['values']['sid']);
  $content_event_handler = get_sunmailer_content_event_handler();
  $content_event_handler->sectionDelete($section);
  SunMailerSection::deleteSection($section->getSectionId());

  // Display a message and redirect.
  drupal_set_message(t('%title successfully deleted', array('%title' => $section->getTitle())));
  $form_state['redirect'] = 'admin/settings/sunmailer/content';
}

/**
 * Displays a list of designs.
 */
function sunmailer_content_overview_designs() {
  $designs = SunMailerDesign::loadAllDesigns();
  $content = 'admin/settings/sunmailer/content';
  $header = array(
    t('Title'),
    t('Type'),
    array('colspan' => 2, 'data' => t('Operations')),
  );
  $rows = array();
  foreach ($designs as $did => $design) {
    $rows[] = array(
      check_plain($design->getTitle()),
      $design->isHtml() ? t('HTML') : t('Text'),
      ($did > 2) ? l('edit', "$content/design/$did/edit") : '',
      ($did > 2) ? l('delete', "$content/design/$did/delete") : '',
    );
  }
  return theme('table', $header, $rows);
}

/**
 * Builds the form for a newsletter design.
 */
function sunmailer_content_design_form($form_state = NULL, $design = NULL) {
  if ($design == NULL) {
    $design = SunMailerDesign::createDesign(TRUE);
  }

  $form = array();
  $form['#attributes'] = array('enctype' => 'multipart/form-data');
  $form['did'] = array(
    '#type' => 'value',
    '#value' => $design->getDesignId(),
  );

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#required' => TRUE,
    '#default_value' => $design->getTitle(),
  );
  $form['html'] = array(
    '#type' => 'radios',
    '#title' => t('Type'),
    '#options' => array(
      TRUE => t('HTML'),
      FALSE => t('Text'),
    ),
    '#required' => TRUE,
    '#default_value' => $design->isHtml(),
  );
  $form['xsl'] = array(
    '#type' => 'file',
    '#title' => t('XSL'),
  );
  if ($design->getXsl()) {
    $form['xsl_filename'] = array(
      '#type' => 'markup',
      '#prefix' => '<p>',
      '#value' => t('Current XSL file: !filename', array('!filename' => $design->getXsl()->filename)),
      '#suffix' => '</p>',
    );
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  return $form;
}

/**
 * Validation callback for XSL file.
 */
function sunmailer_file_validate_is_xsl(&$file) {
  $doc = new DOMDocument();
  return $doc->load($file->filepath) ? array() : array(t('The file is not valid XSL.'));
}

/**
 * Validation callback for the design form.
 */
function sunmailer_content_design_form_validate($form, &$form_state) {
  if ($file = file_save_upload('xsl', array('sunmailer_file_validate_is_xsl' => array()), file_directory_path())) {
    $form_state['values']['xsl_file'] = $file;
  }
  elseif (!$form_state['values']['did']) {
    form_set_error('xsl', t('XSL file field is required.'));
  }
}

/**
 * Submission callback for the design form.
 */
function sunmailer_content_design_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $design = $values['did'] ? SunMailerDesign::loadDesign($values['did']) : SunMailerDesign::createDesign($values['html']);

  // Save the design.
  $design->setTitle($values['title']);
  if (isset($values['xsl_file'])) {
    $design->setXslFileId($values['xsl_file']->fid);
  }
  $design->save();

  // Display a message and redirect.
  if ($values['did']) {
    drupal_set_message(t('Design successfully edited.'));
    $form_state['redirect'] = 'admin/settings/sunmailer/content/design';
  }
  else {
    drupal_set_message(t('Design successfully added.'));
  }
}

/**
 * Confirmation form to delete a design.
 */
function sunmailer_content_design_delete_confirm($form_state = NULL, $design) {
  $form = array();
  $form['did'] = array(
    '#type' => 'value',
    '#value' => $design->getDesignId(),
  );

  $question = t('Are you sure you want to delete the design %title?', array('%title' => $design->getTitle()));
  return confirm_form($form, $question, 'admin/settings/sunmailer/content/design', '', t('Delete'));
}

/**
 * Submission callback for the design deletion form.
 */
function sunmailer_content_design_delete_confirm_submit($form, &$form_state) {
  // Delete the design.
  SunMailerDesign::deleteDesign($form_state['values']['did']);

  // Display a message and redirect.
  drupal_set_message(t('Design successfully deleted.'));
  $form_state['redirect'] = 'admin/settings/sunmailer/content/design';
}

/**
 *
 */
function theme_sunmailer_content_overview_newsletters($form) {
  // Render the table for each newsletter.
  $output = '';
  $list = $form['list']['#value'];
  foreach ($list as $nlid => $sids) {
    drupal_add_tabledrag("sunmailer-newsletter-$nlid", 'order', 'sibling', "sunmailer-newsletter-$nlid-weight");
    $newsletter = $form['content']['newsletter'][$nlid];
    $header = array(
      drupal_render($newsletter['title']),
      drupal_render($newsletter['edit']),
      drupal_render($newsletter['delete']),
      drupal_render($newsletter['add_section']),
      ''
    );
    $rows = array();
    foreach ($sids as $sid) {
      $section = $form['content']['section'][$sid];
      $section['weight']['#attributes']['class'] = "sunmailer-newsletter-$nlid-weight";
      $rows[] = array(
        'class' => 'draggable',
        'data' => array(
          drupal_render($section['title']),
          drupal_render($section['edit']),
          drupal_render($section['delete']),
          '',
          drupal_render($section['weight']),
        ),
      );
    }
    $output .= theme('table', $header, $rows, array('id' => "sunmailer-newsletter-$nlid"));
  }
  // Render the newsletter ordering table.
  drupal_add_tabledrag('sunmailer-newsletter', 'order', 'sibling', 'sunmailer-newsletter-weight');
  $header = array(
    t('Newsletter order'),
    t('Weight'),
  );
  $rows = array();
  foreach ($list as $nlid => $v) {
    $newsletter = $form['content']['newsletter'][$nlid];
    $newsletter['weight']['#attributes']['class'] = "sunmailer-newsletter-weight";
    $rows[] = array(
      'class' => 'draggable',
      'data' => array(
        drupal_render($newsletter['title']),
        drupal_render($newsletter['weight']),
      ),
    );
  }
  $output .= '<hr />';
  $output .= theme('table', $header, $rows, array('id' => 'sunmailer-newsletter'));
  $output .= '<p>' . t('Newsletters will be displayed on the subscription form in the following order.') . '</p>';
  // Render the rest of the form.
  unset($form['content']);
  $output .= drupal_render($form);
  return $output;
}
