<?php

/**
 * Builds the main subscription form.
 */
function sunmailer_subscriber() {
  // Start with an empty subscriber.
  $subscriber = SunMailerSubscriber::loadSubscriber();

  // Check for an authenticated URL.
  $mail = isset($_GET['mail']) ? $_GET['mail'] : '';
  $authenticate = variable_get('sunmailer_authenticate', TRUE);
  if ($mail) {
    $subscriber = SunMailerSubscriber::loadSubscriber($mail);
    if ($authenticate) {
      $hash = isset($_GET['hash']) ? $_GET['hash'] : '';
      if ($hash != $subscriber->getHash()) {
        $subscriber = SunMailerSubscriber::loadSubscriber();
        drupal_set_message(t('Could not load the subscription - invalid hash'), 'warning');
      }
    }
  }

  // Check for an authenticated user.
  global $user;
  if (!$subscriber->getMail() && $user->uid) {
    $subscriber = SunMailerSubscriber::loadSubscriber($user->mail);
  }

  // Build the subscription form.
  $form = sunmailer_subscribe_base_form($subscriber);
  if ($subscriber->getMail() || !$authenticate) {
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save changes'),
    );
  }
  else {
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Request changes'),
    );
  }
  return $form;
}

function sunmailer_subscriber_submit($form, &$form_state) {
  $authenticated = $form_state['clicked_button']['#value'] == t('Save changes');
  sunmailer_subscribe_base_form_submit($form_state['values'], $authenticated);
  if ($authenticated) {
    drupal_set_message('The changes to your subscription have been saved');
  }
  $form_state['redirect'] = '<front>';
}

function sunmailer_subscriber_confirm() {
  // Check that an email address was provided.
  $mail = isset($_GET['mail']) ? $_GET['mail'] : '';
  if (!$mail) {
    drupal_goto('sunmailer/subscribe');
    return;
  }

  // Authenticate the request.
  $subscriber = SunMailerSubscriber::loadSubscriber($mail);
  $hash = isset($_GET['hash']) ? $_GET['hash'] : '';
  if ($hash != $subscriber->getHash()) {
    drupal_set_message(t('Cannot confirm subscription request - invalid hash'), 'warning');
    drupal_goto('sunmailer/subscribe');
    return;
  }

  // Process the request.
  $nlids = isset($_GET['nlids']) ? explode(',', $_GET['nlids']) : array();
  $all_sids = isset($_GET['sids']) ? explode(',', $_GET['sids']) : array();
  $newsletters = SunMailerNewsletter::loadAllNewsletters();
  foreach ($newsletters as $nlid => $newsletter) {
    // Only process certain newsletters.
    if (!in_array($nlid, $nlids)) {
      continue;
    }

    // Set the subscription for this newsletter.
    $sids = array();
    foreach ($newsletter->getSections() as $sid => $section) {
      if (in_array($sid, $all_sids)) {
        $sids[$sid] = $sid;
      }
    }
    $subscriber->setNewsletterSubscription($nlid, $sids);
  }
  $subscriber->save();
  drupal_set_message(t('Your subscription request has been confirmed'));
  drupal_goto('<front>');
}

/**
 * Builds a form to manage a subscriber's subscription.
 */
function sunmailer_subscribe_subscriber($form_state = NULL) {
  // Start at step 1.
  if (!isset($form_state['storage']['step'])) {
    $form_state['storage']['step'] = 1;
  }

  // Display the form for the current step.
  switch ($form_state['storage']['step']) {
  case 1:
    // Select an email address.
    $form['mail'] = array(
      '#type' => 'textfield',
      '#title' => 'Email address',
      '#required' => TRUE,
    );
    $form['continue'] = array(
      '#type' => 'submit',
      '#value' => t('Continue'),
    );
    return $form;

  case 2:
    // Set the subscription.
    $subscriber = SunMailerSubscriber::loadSubscriber($form_state['values']['mail']);
    $form = sunmailer_subscribe_base_form($subscriber);
    $form['cancel'] = array(
      '#type' => 'submit',
      '#value' => t('Cancel'),
    );
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save')
    );
    return $form;
  }
}

function sunmailer_subscribe_subscriber_submit($form, &$form_state) {
  // Check which button was pressed.
  if ($form_state['clicked_button']['#value'] == t('Back')) {
    $form_state['storage']['step']--;
    return;
  }
  else {
    $form_state['storage']['step']++;
  }

  // Proceed to the next step.
  switch ($form_state['storage']['step']) {
  case 2:
    return;

  case 3:
    // Save the subscription changes.
    sunmailer_subscribe_base_form_submit($form_state['values'], TRUE);
    drupal_set_message(t('The changes to the subscription have been saved'));

    // Reset the form.
    unset($form_state['storage']);
    return;
  }
}

/**
 * Builds a form to import subscribers.
 */
function sunmailer_subscribe_import($form_state = NULL) {
  // Start at step 1.
  if (!isset($form_state['storage']['step'])) {
    $form_state['storage']['step'] = 1;
  }
  $storage = $form_state['storage'];

  // Display the form for the current step.
  switch ($storage['step']) {
  case 1:
    // Import the list from a file or text.
    $form = array();
    $form['#attributes'] = array('enctype' => 'multipart/form-data');
    $form['option1'] = array(
      '#type' => 'fieldset',
      '#title' => t('Option 1'),
    );
    $form['option1']['import_file'] = array(
      '#type' => 'file',
      '#title' => t('Submit a file containg a list of email adddresses'),
      '#default_value' => isset($storage['import_file']) ? $storage['import_file'] : NULL,
    );
    $form['option2'] = array(
      '#type' => 'fieldset',
      '#title' => t('Option 2'),
    );
    $form['option2']['import_text'] = array(
      '#type' => 'textarea',
      '#title' => t('Type a list of email addresses'),
      '#default_value' => isset($storage['import_text']) ? $storage['import_text'] : NULL,
    );
    $form['continue'] = array(
      '#type' => 'submit',
      '#value' => t('Continue'),
    );
    return $form;

  case 2:
    // Load all subscribers and email addresses to import.
    $subscription_aggregator = new SunMailerSubscriptionAggregator();
    $subscribers = $subscription_aggregator->getAllSubscribers();
    $all_mails = $form_state['storage']['all_mails'];

    // Determine which email addresses to import.
    if (isset($form_state['storage']['mails'])) {
      $mails = $form_state['storage']['mails'];
    }
    else {
      $mails = $all_mails;
      foreach ($subscribers as $subscriber) {
        if (isset($all_mails[$subscriber])) {
          unset($mails[$subscriber]);
        }
      }
    }

    // Mark any email addresses which are already subscribed.
    foreach ($subscribers as $subscriber) {
        if (isset($all_mails[$subscriber])) {
          $all_mails[$subscriber] = $subscriber . '*';
        }
    }

    // Select from the parsed email addresses.
    $form = array();
    $form['mails'] = array(
      '#type' => 'checkboxes',
      '#title' => t('Verify these email addresses'),
      '#description' => t('*Email addresses is already subscribed; the existing subscription will be overridden'),
      '#options' => $all_mails,
      '#default_value' => $mails,
    );
    $form['back'] = array(
      '#type' => 'submit',
      '#value' => t('Back'),
    );
    $form['continue'] = array(
      '#type' => 'submit',
      '#value' => t('Continue')
    );
    return $form;

  case 3:
    // Select the subscription for all email adddresses imported.
    $subscriber = SunMailerSubscriber::loadSubscriber();
    $newsletters = SunMailerNewsletter::loadAllNewsletters();
    $form['subscription'] = array(
      '#type' => 'markup',
      '#value' => 'Select the subscription for these email addresses',
    );
    $form += sunmailer_subscribe_base_form($subscriber, $newsletters, FALSE);
    foreach ($newsletters as $nlid => $newsletter) {
      $form[$nlid]['subscribe']['#default_value'] = TRUE;
    }

    $form['back'] = array(
      '#type' => 'submit',
      '#value' => t('Back'),
    );
    $form['import'] = array(
      '#type' => 'submit',
      '#value' => t('Import'),
    );
    return $form;
  }
}

function sunmailer_subscribe_import_validate($form, &$form_state) {
  // Only validate the first step.
  if ($form_state['storage']['step'] != 1) {
    return;
  }

  // First try the file option and load the file text.
  if ($file = file_save_upload('import_file')) {
    $contents = file_get_contents($file->filepath);
    if (!$contents) {
      form_set_error('import_file', t('Failed to get the content of the file'));
      return;
    }
    $form_state['values']['import_file_text'] = $contents;
    return;
  }

  // Then check the text option.
  if (!$form_state['values']['import_text']) {
    form_set_error('import_file', t('No email addresses were submitted'));
  }
}

function sunmailer_subscribe_import_submit($form, &$form_state) {
  // Check which button was pressed.
  if ($form_state['clicked_button']['#value'] == t('Back')) {
    $form_state['storage']['step']--;
    return;
  }
  else {
    $form_state['storage']['step']++;
  }

  // Proceed to the next step.
  $values = $form_state['values'];
  switch ($form_state['storage']['step']) {
  case 2:
    // Find the raw input.
    if ($values['import_file_text']) {
      $raw_mails = $values['import_file_text'];
      unset($form_state['storage']['import_text']);
    }
    else {
      $raw_mails = $values['import_text'];
      $form_state['storage']['import_text'] = $values['import_text'];
    }

    // Parse out the email addresses.
    $all_mails = preg_split('/[\s,;]+/', $raw_mails, -1, PREG_SPLIT_NO_EMPTY);
    $form_state['storage']['all_mails'] = array_combine($all_mails, $all_mails);
    unset($form_state['storage']['mails']);
    return;

  case 3:
    // Store the selection of email addresses.
    $form_state['storage']['mails'] = array_filter($values['mails']);
    return;

  case 4:
    // Import the email addresses.
    foreach ($form_state['storage']['mails'] as $mail) {
      $values['mail'] = $mail;
      sunmailer_subscribe_base_form_submit($values, TRUE);
    }
    drupal_set_message(t('The subscribers have been imported'));

    // Reset the form.
    unset($form_state['storage']);
  }
}

/**
 * Builds a form for subscription settings.
 */
function sunmailer_subscribe_settings() {
  $form = array();

  // Display security and privacy settings.
  $form['security'] = array(
    '#type' => 'fieldset',
    '#title' => t('Security and privacy settings'),
  );
  $form['security']['authenticate'] = array(
    '#type' => 'checkbox',
    '#title' => t('Authenticate subscription requests'),
    '#description' => t('Use email verification for anonymous subscription requests'),
    '#default_value' => variable_get('sunmailer_subscribe_authenticate', TRUE),
  );

  // Display user registration settings.
  $form['user'] = array(
    '#type' => 'fieldset',
    '#title' => t('Authenticated user settings'),
  );
  $form['user']['register'] = array(
    '#type' => 'radios',
    '#title' => t('Subscription form on the user registration page'),
    '#options' => array(
      'opt-out' => t('Opt-out of the newsletters'),
      'opt-in' => t('Opt-in to the newsletters'),
      'none' => t('Do not display the subscription form on the user registration page'),
    ),
    '#default_value' => variable_get('sunmailer_subscribe_register', 'opt-out'),
  );
  $form['user']['advanced'] = array(
    '#type' => 'fieldset',
    '#title' => t('Advanced settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['user']['advanced']['register_weight'] = array(
    '#type' => 'weight',
    '#title' => t('Subscription form weight'),
    '#description' => t('Position the subscription form on the user registration page'),
    '#default_value' => variable_get('sunmailer_subscribe_register_weight', 5),
  );
  $form['user']['advanced']['user_category_weight'] = array(
    '#type' => 'weight',
    '#title' => t('Subscription category weight'),
    '#description' => t('Position the subscripton category under the user edit tab'),
    '#default_value' => variable_get('sunmailer_subscribe_user_category_weight', 5)
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );
  return $form;
}

/**
 * Submission callback for the subscription settings callback.
 */
function sunmailer_subscribe_settings_submit($form, &$form_state) {
  $values = $form_state['values'];
  variable_set('sunmailer_subscribe_authenticate', (bool)$values['authenticate']);
  variable_set('sunmailer_subscribe_register', $values['register']);
  variable_set('sunmailer_subscribe_register_weight', (int)$values['register_weight']);
  variable_set('sunmailer_subscribe_user_category_weight', (int)$values['user_category_weight']);
  menu_rebuild();
}

function sunmailer_subscribe_block($form_state = NULL, $delta) {
  // Select the subscriber.
  global $user;
  $subscriber = SunMailerSubscriber::loadSubscriber($user->mail);

  // Select the newsletters.
  if ($delta == 'all') {
    $newsletters = SunMailerNewsletter::loadAllNewsletters();
  }
  else {
    $newsletters = SunMailerNewsletter::loadNewsletter($delta);
  }

  // Display the form.
  $form = array();
  if ($user->uid) {
    $form['mail'] = array(
      '#type' => 'value',
      '#value' => $subscriber->getMail(),
    );
  }
  $form += sunmailer_subscribe_base_form($subscriber, $newsletters, !$user->uid);
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  return $form;
}

function sunmailer_subscribe_block_submit($form, &$form_state) {
  global $user;
  sunmailer_subscribe_base_form_submit($form_state['values'], $user->uid);
}

/**
 * Builds the common part of all subscription forms.
 */
function sunmailer_subscribe_base_form($subscriber, $newsletters = NULL, $include_mail = TRUE) {
  if (!$newsletters) {
    $newsletters = SunMailerNewsletter::loadAllNewsletters();
  }
  if (!is_array($newsletters)) {
    $newsletters = array($newsletters->getNewsletterId() => $newsletters);
  }

  // Build the email address field.
  $form = array();
  $form['#tree'] = TRUE;
  if ($include_mail) {
    if ($subscriber->getMail()) {
      $form['mail'] = array(
        '#type' => 'hidden',
        '#value' => $subscriber->getMail(),
      );
      $form['mail_display'] = array(
        '#type' => 'textfield',
        '#title' => t('Email adddress'),
        '#default_value' => $subscriber->getMail(),
        '#disabled' => TRUE,
      );
    }
    else {
      $form['mail'] = array(
        '#type' => 'textfield',
        '#title' => t('Email adddress'),
        '#required' => TRUE,
      );
    }
  }

  // Build the subscription fields.
  $nlids = array();
  foreach ($newsletters as $nlid => $newsletter)
  {
    // Load the sections and subscription for this newsletter.
    $sections = $newsletter->getSections();
    if (!count($sections)) {
      continue;
    }
    $newsletter_subscription = $subscriber->getNewsletterSubscription($nlid);

    // Check if the user has subscribed to any sections.
    $nlids[] = $nlid;
    $form[$nlid] = array(
      '#type' => 'fieldset',
      '#title' => check_plain($newsletter->getTitle()),
    );
    $form[$nlid]['subscribe'] = array(
      '#type' => 'checkbox',
      '#title' => 'Subscribe',
      '#default_value' => count($newsletter_subscription),
    );

    // For multi-section newsletters, add an option to subscribe to individual sections.
    if (count($sections) > 1)
    {
      $section_options = array();
      foreach ($sections as $sid => $section) {
        $section_options[$sid] = $section->getTitle();
      }
      if (count($sections) == count($newsletter_subscription)) {
        $newsletter_subscription = array();
      }

      $form[$nlid]['options'] = array(
        '#type' => 'fieldset',
        '#title' => t('Subscription options'),
        '#collapsible' => TRUE,
        '#collapsed' => !count($newsletter_subscription),
      );
      $form[$nlid]['options']['sections'] = array(
        '#type' => 'checkboxes',
        '#title' => t('Sections'),
        '#description' => t('Leave blank to subscribe to all sections'),
        '#options' => $section_options,
        '#default_value' => $newsletter_subscription,
      );
    }
    else {
      $form[$nlid]['options']['sections'] = array(
        '#type' => 'value',
        '#value' => array_fill_keys(array_keys($sections), 0),
      );
    }
  }

  $form['nlids'] = array(
    '#type' => 'value',
    '#value' => $nlids,
  );

  return $form;
}

function sunmailer_subscribe_base_form_submit($edit, $authenticated = FALSE) {
  // Load the subscriber.
  $subscriber = SunMailerSubscriber::loadSubscriber($edit['mail']);

  // Update the subscription.
  $nlids = $edit['nlids'];
  $all_sids = array();
  foreach ($nlids as $nlid)
  {
    $sids = array();
    if ($edit[$nlid]['subscribe']) {
      $sections = $edit[$nlid]['options']['sections'];
      $sids = array_keys(array_filter($sections));
      $sids = count($sids) ? $sids : array_keys($sections);
      $sids = array_combine($sids, $sids);
      $all_sids = array_merge($all_sids, $sids);
    }
    $subscriber->setNewsletterSubscription($nlid, $sids);
  }

  // Save these changes or send a confirmation email.
  if ($authenticated) {
    $subscriber->save();
  }
  else {
    // Create the confirmation mail.
    $subject = '!site_name - Newsletter Subscription Request';
    $subject = t($subject, array('!site_name' => variable_get('site_name', '')));
    $body  = 'We received a request to change your email newsletter subscription. ';
    $body .= 'To confirm this change, please use the following link:';
    $body .= "\n\n";
    $body .= '!confirmation-link';
    $confirmation_link = $subscriber->getConfirmationLink($nlids, $all_sids);
    $body = t($body, array('!confirmation-link' => $confirmation_link));

    // Send the mail.
    $mailer = sunmailer_get_mailer();
    sunmailer_send_mail($mailer, array($subscriber->getMail()), $subject, $body, FALSE);
    drupal_set_message(t('A confirmation email has been sent to @mail', array('@mail' => $subscriber->getMail())));
  }
}
